import sys
import os
import sphinxtrap 

extensions = ["sphinxtrap.ext.rawtoc"]
html_theme = 'sphinxtrap'
html_theme_path = [sphinxtrap.get_theme_dir()]
templates_path = ['_templates']
source_suffix = '.rst'
master_doc = 'index'
project = u'testprj'
copyright = u'2014, foo'
version = '0.1'
release = '0.1'
exclude_patterns = []
pygments_style = 'sphinx'
html_static_path = ['_static']
htmlhelp_basename = 'testprjdoc'



import sys, os
from .mocking import unittest, TestSphinxtrap
from .builder import BuildProject

def suite():
    suite = unittest.TestSuite()
    suite.addTests(unittest.makeSuite(TestSphinxtrap))
    suite.addTests(unittest.makeSuite(BuildProject))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=1).run(suite())



import sys, os

__pyver = sys.version_info[0] * 10 + sys.version_info[1]

if __pyver <= 26:
    import unittest2 as unittest
else:
    import unittest

if __pyver < 33: 
    import mock
else:
    from unittest import mock

from docutils.nodes import emphasis, reference
from sphinxtrap.ext.rawtoc import Link, ParseLinks, icon_role, btn_role
from sphinxtrap.ext.rawtoc import build_raw_toctree, html_page_context


def resolve_doctree(*args, **kwargs):
    '''Emulates sphinx/docutils render_partial'''

    path = os.path.dirname(os.path.realpath(__file__))
    with open(os.path.join(path, "test.html")) as fp:
        html = fp.read()
    return html


#Mocking sphinx's stuff
doctree = mock.Mock()
doctree.traverse.side_effect = lambda x: ['index']

builder = mock.Mock()
builder.env.get_doctree.side_effect = lambda x: doctree
builder.env.resolve_doctree.side_effect = resolve_doctree
builder.render_partial.side_effect = lambda x: {'fragment':resolve_doctree()}


inliner = mock.Mock()
inliner.problematic= mock.Mock()
inliner.reporter= mock.Mock()
inliner.reporter.error = mock.Mock()

class TestSphinxtrap(unittest.TestCase):

    def test_node(self):
        lk = Link(text="gabbagabbahey", href="https://foo.com")
        lk2 = Link(text="gabbagabbahey", href="https://foo.com", depth=1)
        lk["childs"].append(lk2)
        self.assertEqual(lk['childs'][0].get("current"), False)
        self.assertEqual(lk['href'], "https://foo.com")

    def test_parser(self):
        pl = ParseLinks()
        path = os.path.dirname(os.path.realpath(__file__))
        with open(os.path.join(path, "test.html")) as fp:
            pl.feed(fp.read())
        self.assertEqual(pl.links[7]['childs'][0]['href'],
                "#compression-algorithms")
    
    def test_build_raw_toctree(self):
        links = build_raw_toctree(builder, "test", False, False)
        self.assertEqual(links[7]['childs'][0]['href'],
                "#compression-algorithms")

    def test_icon_role(self):
        icon = icon_role("foo", ":icon:`foo`", "foo", 100,
                inliner, options={'classes':[]}, content=[])
        self.assertTrue(isinstance(icon[0][0], emphasis ))

    def test_btn_role(self):
        l_str = ":btn:`Foo <http://foo.org/>,btn-success,icon-globe`"
        btn = btn_role("foo", l_str, l_str, 100, inliner,
                options={'classes':[]}) 
        self.assertTrue(isinstance(btn[0][0], reference))


    def test_html_page_context(self):
        global context
        context = {}
        app = mock.Mock()
        app.builder = builder
        html_page_context(app, 'index', 'foo', context, None)
        local = context['raw_toctree']()
        toc = context['raw_localtoc']()
        self.assertFalse(toc[0]['current'])
        self.assertFalse(local[0]['current'])



import os

__version__ = "0.2"
__release__ = "{0}.{1}".format(__version__, "1") 

_root_dir = os.path.abspath(os.path.dirname(__file__))

def get_theme_dir():
    return os.path.join(_root_dir, "themes")



import os
try:
    from StringIO  import StringIO
except ImportError:
    from io import StringIO
import tempfile
from shutil import rmtree

from sphinx import cmdline

from . mocking import unittest, mock


global cache 
cache = {'tmp_file':'/tmp/__sphinxtrap__',
        'index_html': None} 


class BuildProject(unittest.TestCase):

    index_html = None

    def test_1_build(self):
        cache['old_pwd'] = os.getcwd() 
        path = os.path.dirname(os.path.realpath(__file__))
        os.chdir(os.path.join(path, "testprj"))
        cache['tmp_file'] = tempfile.mkdtemp()

        args = ['sphinx-build', '-b', 'html', '-d', 
                cache['tmp_file'] + '/doctrees', 'source', 
                cache['tmp_file'] + '/build/html']
        with mock.patch('sys.stdout', new=StringIO()) as fake_out: 
            retv = cmdline.main(args)
        self.assertEqual(retv, 0)

    def read(self):
        if cache['index_html'] is None:
            fname = cache['tmp_file'] + '/build/html/index.html'
            with open(fname, 'rt') as fd:
                cache['index_html'] = fd.read()
        return cache['index_html']


    def test_2_icon_role(self):
        btn = '<p><em class="icon-bullhorn icon-holder"></em></p>'
        self.assertIn(btn, self.read())

    def test_2_btn_role(self):
        btn = '<a class="btn reference external" ''href="http://sphinx-doc.or'
        'g/"><em class="icon-globe icon-holder"></em> Sphinx </a>'
        self.assertIn(btn, self.read())

    def test_3_tear_down(self):
        rmtree(cache['tmp_file'])
        os.chdir(cache['old_pwd'])





#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys,os
from os.path import abspath, join
root_path = abspath(join(__file__, ".."))
os.chdir(root_path)
lib_path = '.'

from setuptools import setup, find_packages

from sphinxtrap import __release__ as VERSION

setup(
    packages = find_packages(),
    package_data = { "sphinxtrap": ["themes/*/*.*", "themes/*/static/*.*",
        "themes/*/static/img/*", "themes/*/static/font/*"]},
    zip_safe=False,
    install_requires=[ "sphinx>=1.1"],
    name = "sphinxtrap",
    version = VERSION,
    author = "José Manuel Fardello",
    author_email = "jmfardello@gmail.com",
    description = "Yet another bootstrap sphinx theme.",
    long_description="""\
Small package containing a Sphinx theme named "sphinxtrap",
You can see an example at `<//http://jfardello.github.com/Dardrive/>`_.
    """,
    license = "MIT",
    keywords = "sphinx extension theme",
    url = "https://github.com/jfardello/Sphinxtrap",
    download_url = "http://pypi.python.org/pypi/sphinxtrap",
    test_suite = "tests.suite",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Documentation',
        'Topic :: Software Development :: Documentation',
    ]
)



import sys

try:
    from html.parser import HTMLParser, HTMLParseError
except ImportError:
    from  HTMLParser import HTMLParser, HTMLParseError
import logging
import re

from sphinx import addnodes
from docutils.nodes import emphasis, reference, Text
from docutils.parsers.rst.roles import set_classes


class Link(dict):

    '''Minimal nested link representation.'''

    def __init__(self, *args, **kwds):
        default = {'href': None, 'text': None, 'current': False,
                   'depth': None, 'childs': []}
        default.update(kwds)
        self.update(*args, **default)


class ParseLinks(HTMLParser):

    '''Parses sphinx's toctree html output into a list of nested Link
    objects, to simplify the bootstrap stuff inside Jinja templates.'''

    def __init__(self):
        self.logger = logging.getLogger("%s.%s" %
                (__name__, self.__class__.__name__))
        HTMLParser.__init__(self)

        self.depth = 0
        self.tag_stack = []
        self.links = []
        self.current = False
        self._tag = None
        self._tmp = None
        self._dataq = ""

    def handle_starttag(self, tag, attrs):
        if tag == 'ul':
            self.logger.debug("adding %s to tag_stack" % tag)
            self.tag_stack.append(tag)
            self.depth += 1

        if tag == 'li':
            self.logger.debug("adding %s to tag_stack" % tag)
            self.tag_stack.append(tag)
            for name, value in attrs:
                if name == 'class' and 'toctree-l' in value:
                    classes = value.split()
                    self.depth = int(classes[0].replace('toctree-l', ''))
                    if 'current' in classes:
                        self.current = True
                    else:
                        self.current = False
        if tag == 'a':
            self.logger.debug("adding %s to tag_stack" % tag)
            self.tag_stack.append(tag)
            self._tag = tag
            for name, value in attrs:
                if name == 'href':
                    href = value
            self._tmp = (Link(href=href, depth=self.depth,
                              current=self.current))

    def handle_data(self, data):
        if self._tag == 'a':
            data = " ".join(data.split())
            self.logger.debug("handle 'a tag' data: %s" % data)
            if self._tmp['text'] is None:
                seen = False
                self._tmp['text'] = data
            else:
                seen = True
                self._tmp['text'] += " " + data
            if not seen:
                if self._tmp['depth'] > 1:
                    links = self.links
                    loop = 0

                    for pos in range(self._tmp['depth']):
                        if loop == 0:
                            links = links[-1:]
                        else:
                            links = links[-1:][0]['childs']
                        loop += 1
                    links.append(self._tmp)
                else:
                    self.links.append(self._tmp)

    def handle_endtag(self, tag):
        try:
            if tag in ["li", "a"] and tag == self.tag_stack[-1:][0]:
                self.logger.debug("poping %s from stack" % tag)
                self.tag_stack.pop()
            self._tag = None
            if tag == 'ul' and tag == self.tag_stack[-1:][0]:
                self.tag_stack.pop()
                self.depth -= 1
        except IndexError:
            raise HTMLParseError('Unbalanced html tags.')


def html_page_context(app, pagename, templatename, context, doctree):
    """ Handler for the html-page-context signal, adds a raw_toctree function
    to the context."""

    def raw_toctree(collapse=False):
        return build_raw_toctree(app.builder, pagename, prune=False,
                                 collapse=collapse)

    def raw_localtoc(collapse=False):
        self_toc = app.builder.env.get_toc_for(pagename, app.builder)
        toc = app.builder.render_partial(self_toc)['fragment']
        pl = ParseLinks()
        pl.feed(toc)
        try:
            index = pl.links[0]
            childs = list(index['childs'])
            index['childs'] = []
            childs.insert(0, index)
            return childs
        except IndexError:
            return []

    context['raw_toctree'] = raw_toctree
    context['raw_localtoc'] = raw_localtoc


def build_raw_toctree(builder, docname, prune, collapse):
    """ Returns a list of nested Link objects representing the toctree."""
    env = builder.env
    doctree = env.get_doctree(env.config.master_doc)
    toctrees = []
    for toctreenode in doctree.traverse(addnodes.toctree):
        toctree = env.resolve_toctree(docname, builder, toctreenode,
                                      prune=prune, collapse=collapse, 
                                      includehidden=True)
        toctrees.append(toctree)
    if not toctrees:
        return None
    retv = toctrees[0]
    for toctree in toctrees[1:]:
        if toctree:
            retv.extend(toctree.children)
    pl = ParseLinks()
    pl.feed(builder.render_partial(retv)['fragment'])

    return pl.links


def icon_role(role, rawtext, text, lineno, inliner, options={}, content=[]):

    options.update({'classes': ["icon-" + x for x in text.split(",")]})
    options['classes'].append('icon-holder')
    set_classes(options)
    node = emphasis(**options)
    return [node], []


def btn_role(role, rawtext, text, lineno, inliner, options={}, content=[]):

    m = re.match(r'(.*)\<(.*)\>(,.*){0,}', text)
    if m:
        name, ref, classes = m.groups()
        if ref is '':
            msg = inliner.reporter.error('The ref portion of the btn role'
                                         ' cannot be none')
            prb = inliner.problematic(rawtext, rawtext, msg)
            return [prb], [msg]

        ref_cls = []
        em_cls = []
        if classes:
            cls = classes.strip(',').split(',')
            em_cls = [x for x in cls if x.startswith("icon-")]
            ref_cls = [x for x in cls if not x.startswith("icon-")]
            if 'btn' not in ref_cls:
                ref_cls.append('btn')
            options.update({'classes': ref_cls})


        set_classes(options)
        node = reference(rawtext, name, refuri=ref, **options)
        if len(em_cls) > 0:
            em_cls.append('icon-holder')
            em_opts = {'classes': em_cls}
            set_classes(em_opts)
            node.insert(0, emphasis(**em_opts))
            node.insert(1, Text(" "))

        return [node], []


def setup(app): # pragma: no cover
    app.info('Adding the icon role')
    app.add_role('icon', icon_role)
    app.info('Adding the btn role')
    app.add_role('btn', btn_role)
    app.info('Adding the raw_toctree Jinja function.')
    app.connect('html-page-context', html_page_context)
    return
